<?php
/**
 * Omega Commerce
 *
 * Licence: MIT https://opensource.org/licenses/MIT
 * Copyright: 2017 Omega Commerce LLC https://omegacommerce.com
 */
// if you use wp multisite, uncomment the following line and enter your info.
// $_SERVER['HTTP_HOST'] = 'your_host.com';
// $_SERVER['REQUEST_URI'] = '/your_wp_folder/';

if (php_sapi_name() !== 'cli') {
    die("you can run this script only via shell");
}

if ( !defined('ABSPATH') ) {
    /** Set up WordPress environment */
    require_once( dirname( __FILE__ ) . '/../../../../wp-load.php' );
}

function progressBar($i, $total) {
    if ($total == 0) {
        $perc = 100;
    } else {
        $perc = floor(($i / $total) * 100);
    }
    $left = 100 - $perc;
    $write = sprintf("\033[0G\033[2K[%'={$perc}s>%-{$left}s] $perc%% - $i/$total", "", "");
    fwrite(STDERR, $write);
}

$quiet = false;
$full = false;
$entityType = false;

$indexer = $omegaManager->getApiIndexer();
$entityList = $indexer->getEntities();
$limit = get_option('omega_api_max_sync_number');
$single = false;
$clean = false;

foreach($argv as $i => $arg) {
    if ($i == 0) {
        continue;
    }
    if ($arg == "--quiet") {
        $quiet = true;
    } elseif ($arg == "--full") {
        $full = true;
    } elseif ($arg == "--list-entities") {
        echo "Listing available entities for reindex: " . PHP_EOL;
        foreach ($entityList as $entity) {
            echo " -- " . $entity->getType() . PHP_EOL;
        }
        die;
    } elseif ($arg == "--entity") {
        $entityType = $argv[++$i];
        continue;
    } elseif ($arg == "--sync-limit") {
        $limit = $argv[++$i];
        continue;
    } elseif ($arg == "--single-batch") {
        $single = true;
        continue;
    } elseif ($arg == "--clean") {
        $clean = true;
        continue;
    } elseif ($arg == "--help") {
        echo "Available options:\n\n";
        echo "--clean - clean all index data\n";
        echo "--list-entities - list available entities\n";
        echo "--entity - reindex only specified entity type\n";
        echo "--sync-limit - specify processing bulk size\n";
        echo "--single-batch - execute only one batch (used for weak servers)\n";
        echo "--quiet - hide progress\n";
        echo "--full - clean indexes and run reindex\n";
        echo "\n";
        die;
    }
}

// Remove all records both from remote and local indices
if ($full || $clean) {
    $tries = 1;
    while (true) {
        try {
            $indexer->clean();
            break;
        } catch (\OmegaCommerce\Api\Exception $e) {
            $write = sprintf("\033[0G\033[2K%'= A" . $e->getMessage() . " Trying again ($tries)...", "", "");
            fwrite(STDERR, $write);
            sleep(5);
            $tries++;
            if ($tries > 10) {
                exit(1);
            }
            continue;
        }
    }
}

if($clean && !$full) {
    fwrite(STDERR, "\033[32m" . "Index cleanup is completed!\n");
    die;
}

$i = 1;
$tries = 1;

delete_option(\OmegaCommerce\Model\Config::NOTICE_FLAG_ASK_REINDEX);
update_option('omega_cron_last_at', gmdate('U'));

foreach ($entityList as $entity) {
    if($entityType && ($entity->getType() != $entityType)) {
        fwrite(STDERR, "\033[31m" . "Skipping " . $entity->getHumanName() . "...\n\033[0m");
        continue;
    }

    if (!$quiet) {
        fwrite(STDERR, "\033[31m" . "Reindexing " . $entity->getHumanName() . "...\n\033[0m");
    }

    // Write to database list of changed (or all - if database was cleaned) entities
    $n = $indexer->refreshStatus($entity);

    // Remove non-existing items from database (use specified sync limit as batch size limit)
    while (true) {
        try {
            $count = $indexer->removeEntity($entity, $limit);
            if ($count == 0) {
                break;
            }
        } catch (\OmegaCommerce\Api\Exception $e) {
            $write = sprintf("\033[0G\033[2K%'= A" . $e->getMessage() . " Trying again ($tries)...", "", "");
            fwrite(STDERR, $write);
            sleep(5);
            $tries++;
            if ($tries > 10) {
                exit(1);
            }
            continue;
        }
        $tries = 1;
    }


    // Get number of items, that needs to be reindexed
    $totalNumber = $indexer->reindexQueueLength($entity);
    if (!$quiet) {
        echo "Reindex queue size: " . $totalNumber . PHP_EOL;
        echo "Using sync limit: " . $limit . PHP_EOL;
    }

    if ($totalNumber == 0 && $entity->getMainTable()) {
        if (!$quiet) {
            fwrite(STDERR, "Everything is updated\n");
        }
        continue;
    } elseif (!$entity->getMainTable()) {
        if (!$quiet) {
            fwrite(STDERR, "No items of this content type\n");
        }
        continue;
    }

    // Reindex items using specified sync limit
    while (true) {
        try {
            $indexer->reindexEntity($entity, $limit);
        } catch (\OmegaCommerce\Api\Exception $e) {
            $write = sprintf("\033[0G\033[2K%'= A" . $e->getMessage() . " Trying again ($tries)...", "", "");
            fwrite(STDERR, $write);
            sleep(5);
            $tries++;
            if ($tries > 10) {
                exit(1);
            }
            continue;
        }
        $tries = 1;
        $currentTotal = $i * $limit;
        if ($currentTotal > $totalNumber) {
            $currentTotal = $totalNumber;
        }
        if (!$quiet) {
            progressBar($currentTotal, $totalNumber);
        }
        if ($currentTotal >= $totalNumber || $single) {
            break;
        }
        $i++;
    }
    if (!$quiet) {
        fwrite(STDERR, "\n");
    }
}
if (!$quiet) {
    fwrite(STDERR, "\033[32m" . "Reindexing is completed!\n");
}