<?php
/**
 * Omega Commerce
 *
 * Licence: MIT https://opensource.org/licenses/MIT
 * Copyright: 2017 Omega Commerce LLC https://omegacommerce.com
 */
namespace OmegaCommerce\Controller\Admin;

class SyncController
{
    protected $results, $errorMessage, $errorCode, $stepN, $operation, $entityIndex, $total, $currentTotal;

    /**
     * @var \OmegaCommerce\Api\Interfaces\EntityInterface
     */
    protected $nextEntity;

    public function __construct(
        \OmegaCommerce\Api\Config $config,
        \OmegaCommerce\Api\Auth $auth,
        \OmegaCommerce\Api\Indexer $indexer
    )
    {
        $this->config = $config;
        $this->auth = $auth;
        $this->indexer = $indexer;

        add_action('admin_enqueue_scripts', function($hook){
            if ('admin_page_omega_commerce_sync' == $hook) {
                wp_enqueue_script('omega_sync_script',
                    sprintf("%ssrc/view/js/sync.js", WP_OMEGA_COMMERCE_PLUGIN_URL), array(), false, true);
                wp_enqueue_style('omega_sync_css',
                    sprintf("%ssrc/view/css/sync.css", WP_OMEGA_COMMERCE_PLUGIN_URL));
            }
        });

        wp_enqueue_style('omega_sync_status_css',
            sprintf("%ssrc/view/css/sync-status.css", WP_OMEGA_COMMERCE_PLUGIN_URL));
        add_action('wp_ajax_omega_data_sync', array(&$this, 'syncAction'));
    }

    public function step($num, $entityIndex, $action) {
        return $num."_".$entityIndex."_".$action;
    }

    /**
     * AJAX callback handler
     */
    public function syncAction()
    {
        if (isset($_GET['d'])) {
            error_reporting(E_ALL);
            ini_set('display_errors', 1);
            global $wpdb;
            $wpdb->show_errors(true);
            $wpdb->suppress_errors(false);
        }
        $nonce = $_REQUEST['step_nonce'];
        $this->stepN = (int)$_REQUEST['step'];
        $this->operation = $_REQUEST['operation'];
        $this->entityIndex = $_REQUEST['entity'];

        if (!wp_verify_nonce($nonce, 'omega-commerce-sync-nonce')) {
            echo '';
            wp_die();
        }
        if(!session_id()) {
            session_start();
        }
        global $_SESSION;

        delete_option(\OmegaCommerce\Model\Config::NOTICE_FLAG_ASK_REINDEX);

        try {
            $this->run();
        } catch(\OmegaCommerce\Api\Exception $e) {
            $this->errorCode = $e->getCode();
            $this->errorMessage = $e->getMessage();
            status_header($this->errorCode);
        }
        ob_start();
        include(sprintf("%ssrc/view/templates/syncResult.php", WP_OMEGA_COMMERCE_PLUGIN_PATH));
        $html = ob_get_contents();
        ob_end_clean();

        echo json_encode(array(
            "html" => $html,
            "next_step" => $this->stepN,
            "next_operation" => $this->operation,
            "next_entity" => $this->entityIndex,
        ));

        wp_die();
    }

    const ACTION_INIT = "init";
    const ACTION_CLEANUP = "cleanup";
    const ACTION_REFRESH_STATUS = "collecting_changes";
    const ACTION_REINDEX = "reindex";
    const ACTION_FINISH = "finish";

    public function run()
    {
        $custom_reindex_params = (get_transient('custom_reindex_params')) ? get_transient('custom_reindex_params') : NULL;
        $entities = array();

        foreach($this->indexer->getEntities() as $entity) {

            // Custom Reindex with specific entities select
            if($custom_reindex_params && !in_array($entity->getType(), $custom_reindex_params["content"])) {
                continue;
            }
            $entities[$entity->getType()] = $entity;
        }
        if ($this->operation == self::ACTION_INIT) {
            $this->operation = self::ACTION_REFRESH_STATUS;
            $this->stepN = 1;
            $this->entityIndex = array_keys($entities)[0];
            $this->currentTotal = -1;
            $this->total = -1;
        }

        $limit = get_option('omega_api_max_sync_number');

        // Custom Reindex with limit override
        if($custom_reindex_params && isset($custom_reindex_params["limit"])) {
            $limit = $custom_reindex_params["limit"];
        }

        $entity = $entities[$this->entityIndex];

        if ($this->operation == self::ACTION_REFRESH_STATUS) {
            $count = $this->indexer->refreshStatus($entity);
            $this->operation = self::ACTION_CLEANUP;
        }
        if ($this->operation == self::ACTION_CLEANUP) {
            $count = $this->indexer->removeEntity($entity, $limit);
            if ($count == 0) {
                $this->operation = self::ACTION_REINDEX;
            }
        }

        if ($this->stepN == 1) {
            $_SESSION["omega_total"] = $this->indexer->reindexQueueLength($entity);
        }
        $this->nextEntity = $entity;

        if ($this->operation == self::ACTION_REINDEX) {
            $this->indexer->reindexEntity($entity, $limit);

            // Custom Reindex in single-batch mode
            if($custom_reindex_params && isset($custom_reindex_params["single"])) {
                $count = $this->indexer->refreshStatus($entity);
                $count = $this->indexer->removeEntity($entity, $limit);
                $this->operation = self::ACTION_FINISH;
                delete_transient('custom_reindex_params');
                return;
            }

            $this->total = $_SESSION["omega_total"];
            $this->currentTotal = $this->stepN * $limit;

            //30 * 27 > 800 && 30 * 27 < 800 + 30
            if ($this->currentTotal > $this->total && $this->currentTotal < $this->total + $limit) {
                $this->currentTotal = $this->total;
                $this->stepN++;
                $this->nextEntity = $entity;
                //30 * 28 > 800
            } else if ($this->currentTotal > $this->total) {
                $this->stepN = 1;
                $this->nextEntity = false;
                do {
                    $current = current($entities);
                    if ($this->entityIndex == $current->getType()) {
                        next($entities);
                        $current = current($entities);
                        if ($current) {
                            $this->entityIndex = $current->getType();
                            $this->nextEntity = $current;
                        }
                        break;
                    }
                } while(next($entities));

                if ($this->nextEntity) {
                    $this->operation = self::ACTION_REFRESH_STATUS;
                } else {
                    $this->operation = self::ACTION_FINISH;
                    delete_transient('custom_reindex_params');
                }
            } else {
                $this->nextEntity = $entity;
                $this->stepN++;
            }
        }
    }

    /**
     *  Show sync page handler
     */
    public function showAction()
    {
        if (!$this->auth->isAuthorized()) {
            $this->auth->register(get_site_url());
        }

        if (isset($_GET["clear"])) {
            try {
                $this->indexer->clean();
            } catch(\OmegaCommerce\Api\Exception $e) {
                echo "</br>";
                echo $e->getMessage();
                die;
            }
        }
        if (isset($_GET["content"])) {
            $custom_reindex_params = json_decode(json_encode($_GET), true);
            $custom_reindex_params["content"] = explode(" ", $custom_reindex_params["content"]);
            set_transient('custom_reindex_params', $custom_reindex_params, 12 * HOUR_IN_SECONDS);
        }
        if (!current_user_can(\OmegaCommerce\Model\Menu::OMEGA_COMMERCE_CAPABILITY)) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        include(sprintf("%ssrc/view/templates/syncPage.php", WP_OMEGA_COMMERCE_PLUGIN_PATH));
    }

    /**
     *  Show sync page handler
     */
    public function showButtonAction()
    {
        if (!current_user_can(\OmegaCommerce\Model\Menu::OMEGA_COMMERCE_CAPABILITY)) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        if (!$this->auth->isAuthorized()) {
            $this->auth->register(get_site_url());
        }
        if (isset($_GET["clear"])) {
            try {
                $this->indexer->clean();
            } catch(\OmegaCommerce\Api\Exception $e) {
                echo "</br>";
                echo $e->getMessage();
                die;
            }
            echo '<br><div class="update-nag" style="border-left: 4px solid #008000;width: 100%;">Indexes are empty. Please, run data synchronisation.</div>';
        }

        if (isset($_GET["restore_table"])) {
            try {
                $this->restoreDatabaseTable();
            } catch(\OmegaCommerce\Api\Exception $e) {
                echo "</br>";
                echo $e->getMessage();
                die;
            }
            echo '<br><div class="update-nag" style="border-left: 4px solid #008000;width: 100%;">Omega Instant Search Index Table is restored. Please, run data synchronisation.</div>';
        }

        include(sprintf("%ssrc/view/templates/syncButtonPage.php", WP_OMEGA_COMMERCE_PLUGIN_PATH));
    }

    public function restoreDatabaseTable()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . \OmegaCommerce\Api\Indexer::OMEGA_SYNC_STATUS_TABLE;
        $charset_collate = "DEFAULT COLLATE " . $this->indexer->getDBCollation();
         $sql = "CREATE TABLE `$table_name` (
  `entity_id` bigint(20) NOT NULL,
  `entity_type` varchar(25) NOT NULL,
  `table_name` varchar(25) NOT NULL DEFAULT '',
  `updated_at` TIMESTAMP NOT NULL ON UPDATE CURRENT_TIMESTAMP,
  `hash` TEXT DEFAULT NULL,
  `old_hash` TEXT DEFAULT NULL,
  `is_reindexed` tinyint(1) NOT NULL DEFAULT '0',
  PRIMARY KEY (`entity_id`,`entity_type`,`table_name`)
        ) $charset_collate;";

        $wpdb->query($sql);
    }

    public function getDatabaseTable()
    {
        global $wpdb;
        $omegaTable = $wpdb->prefix . \OmegaCommerce\Api\Indexer::OMEGA_SYNC_STATUS_TABLE;
        $sql = 'SHOW TABLES LIKE "' . $omegaTable . '"';
        if($wpdb->query($sql)) {
            return $omegaTable;
        }

        return false;
    }

    public function getLastReindexed()
    {
        global $wpdb;
        $omegaTable = $wpdb->prefix . \OmegaCommerce\Api\Indexer::OMEGA_SYNC_STATUS_TABLE;
        $sql = 'SELECT DISTINCT updated_at FROM ' . $omegaTable . ' WHERE updated_at IN (SELECT max(updated_at) FROM ' . $omegaTable . ')';
        if($result = $wpdb->get_results($sql)) {
            $result = json_decode(json_encode($result), true); // WPDB returns StdClass, so we need to convert
            return $result[0]['updated_at'];
        }

        return false;
    }

    public function getIndexEntries($entryType = "*", $entrySubtype = "*", $reindexed = true)
    {
        global $wpdb;
        $omegaTable = $wpdb->prefix . \OmegaCommerce\Api\Indexer::OMEGA_SYNC_STATUS_TABLE;
        $sql = 'SELECT COUNT(*) FROM ' . $omegaTable;
        if($entryType != "*") {
            $sql .= ' WHERE entity_type = "' . $entryType . '"';
            if($entrySubtype != "*") {
                $sql .= ' AND table_name = "' . $wpdb->prefix . $entrySubtype  . '"';
            }
            if($reindexed) {
                $sql .= ' AND is_reindexed = 1';
            } else {
                $sql .= ' AND is_reindexed = 0';
            }
        }
        if($result = $wpdb->get_results($sql)) {
            $result = json_decode(json_encode($result), true); // WPDB returns StdClass, so we need to convert
            return $result[0]['COUNT(*)'];
        }
        return 0;
    }

    public function formatEntryIndexStatus($entryType = "*", $entrySubtype = "*")
    {
        $indexStatus = strval($this->getIndexEntries($entryType, $entrySubtype));
        $unindexed = $this->getIndexEntries($entryType, $entrySubtype, false);
        if($unindexed > 0) {
            $indexStatus .= " (" . $unindexed . " in queue)";
        }
        return $indexStatus;
    }

    public function checkWordpressConnection()
    {
        $data = array(
            "secretKey" => $this->config->getSecretKey(),
            "product" => "search",
            "domain" => $_SERVER['HTTP_HOST'],
            "URL" => get_site_url(),
            "platform" => "woocommerce");

        $result = wp_remote_post("https://search.omegacommerce.com/store/api/v2/woocommerce/auth/register?version=_VERSION_&ID=" . $this->config->getID() . "&key=" . $this->config->getSecretKey(), array(
                'method' => 'POST',
                'timeout' => 45,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => array(
                    'Content-type: application/json',
                    ),
                'body' => json_encode($data),
                'cookies' => array()
                )
        );

        if($result["response"]["code"] == 200) {
            return true;
        }
        return false;
    }

    public function getAvailableEntities()
    {
        return $this->indexer->getEntities();
    }
}