<?php
/**
 * Omega Commerce
 *
 * Licence: MIT https://opensource.org/licenses/MIT
 * Copyright: 2017 Omega Commerce LLC https://omegacommerce.com
 */

namespace OmegaCommerce\Entity\Helper;

class Product
{
    public function __construct(
        \OmegaCommerce\Entity\Helper\Render $renderHelper,
        \OmegaCommerce\Entity\Helper\Post $postHelper
    )
    {
        $this->renderHelper = $renderHelper;
        $this->postHelper = $postHelper;
    }

    /**
     * @param \WP_Post $post
     * @return array
     */
    public function getData($post) {
        $data = $this->postHelper->getData($post);
        $product = wc_get_product($post->ID);

        // Add WooCommerce Product Tags as Tags
        if($terms = get_the_terms($post->ID, 'product_tag')) {
            $productTags = array_column($terms, 'name');

            if($productTags) {
                if(is_array($data['tags'])) {
                    $data['tags'] = array_merge($data['tags'], $productTags);
                } else {
                    $data['tags'] = $productTags;
                }
            } else {
                $data['tags'] = array();
            }
        }

        return array_merge($data, $this->getProductData($product));
    }

    /**
     * @param \WC_Product $product
     * @return array
     */
    private function getProductData($product, $grouped = false)
    {
        $data = array();
        if (method_exists($product, 'get_catalog_visibility')) { //woocommerce  2.4.13 does not have this function
            $visability = $product->get_catalog_visibility();
        } else {
            $visability = $product->visibility;
        }

        if ($visability != "search" && $visability != "visible" && !$grouped) {
            return false;
        }
        $data['sku'] = $product->get_sku();

        $data['final_price'] = $this->getProductDisplayPrice($product, $product->get_sale_price());
        $data['regular_price'] = $this->getProductDisplayPrice($product, $product->get_regular_price());
        $data['price'] = $this->getProductDisplayPrice($product, $product->get_price());
        $attributes = $this->getAttributes($product);
        if (count($attributes)) { //if empty, our backend fails with error
            $data['attributes'] = $attributes;
        }

        if ($this->getProductType($product) == 'variable') {
            $variations = $this->get_variations($product);
            if (count($variations)) {
                $data['variants'] = $variations;
            }
        }

        if ($this->getProductType($product) == 'grouped') {
            $groupProducts = $this->get_group_products($product);
            if(count($groupProducts)) {
                $data['grouped_products'] = $groupProducts;
            }
        }

        $data['store_id'] = get_current_blog_id();
        $data['is_in_stock'] = ((int)$product->is_in_stock()) + 1;//0 - unknown, 1 - out of, 2 - in stock

        return $data;
    }

    /**
     * @param \WC_Product $product
     * @return string
     */
    private function getProductType($product) {
        if (method_exists($product, "get_type")) {
            return $product->get_type();
        }
        return $product->product_type;
    }

    /**
     * @param \WC_Product $product
     * @param float $price
     * @return float
     */
    private function getProductDisplayPrice($product, $price)
    {
        if (function_exists("wc_get_price_to_display")) {//woo 3.0
            return wc_get_price_to_display($product,  array(
                'qty'   => 1,
                'price' => $price,
            ) );
        }

        if (method_exists($product, 'get_display_price')) {
            return $product->get_display_price($price);
        }

        $taxDisplayMode = get_option('woocommerce_tax_display_shop');

        return $taxDisplayMode == 'incl' ? $product->get_price_including_tax() : $product->get_price_excluding_tax();
    }

    /**
     * @param \WC_Product $product
     * @return array
     */
    private function get_variations($product)
    {
        $variations = $product->get_available_variations();
        $data = array();
        foreach ($variations as $variation) {
            $item = array(
                'variation_id' => $variation['variation_id'],
                'sku' => $variation['sku'],
                'is_in_stock' => $variation['is_in_stock'],
                'description' => $variation['variation_description'],
                'v2_price' => $variation['price'],
                'regular_price' => $variation['regular_price'],
                'sale_price' => $variation['sale_price'],
                'stock_quantity' => $variation['stock_quantity'],
            );
            if (isset($variation['display_price'])) {
                $item['v2_price'] = $variation['display_price'];
            }
            if (isset($variation['display_regular_price'])) {
                $item['regular_price'] = $variation['display_regular_price'];
            }
            if (count($variation['attributes'])) { //if empty, our backend fails with error
                $item['attributes'] = $variation['attributes'];
            }
            $data[] = $item;
        }

        return $data;
    }

    /**
     * @param \WC_Product $product
     * @return array
     */
    private function get_group_products($product)
    {
        $productIds = $product->get_children();
        $data = array();

        foreach ($productIds as $productId) {
            $groupProduct = wc_get_product($productId);
            $productData = $this->getProductData($groupProduct, true);
            $productData['post_id'] = $productId;
            $productData['post_title'] = method_exists($product, "get_name") ? $groupProduct->get_name() : $groupProduct->name;
            $productData['description'] = method_exists($product, "get_description") ? $groupProduct->get_description() : $groupProduct->description;
            $data[] = $productData;
        }
        return $data;
    }

    /**
     * @param \WC_Product $product
     * @return array
     */
    private function getAttributes($product)
    {
        if (!defined('WC_DELIMITER')) {
            $delimiter = '|';
        } else {
            $delimiter = WC_DELIMITER;
        }
        if (method_exists($product, "get_id")) {
            $productId = $product->get_id();
        } else {
            $productId = $product->id; //woocommerce Version 2.4.13
        }

        $attributes = $product->get_attributes();
        $data = array();
        foreach ($attributes as $attribute) {
            $code = $attribute['name'];
            if ($attribute['is_taxonomy']) {
                $value = wc_get_product_terms($productId, $code, array('fields' => 'names'));
            } else {
                $value = explode($delimiter, $attribute['value']);
            }
            $data[$code] = $value;
        }

        $thirdParty = array(
            "product_tag",
            "product_brand",
        );
        foreach ($thirdParty as $code) {
            $value = wc_get_product_terms($productId, $code, array('fields' => 'names'));
            $data[$code] = $value;
        }
        return $data;
    }
}
